//=============================================================================
//  Unified IO - C++ interface for industrial IO cards
//  Copyright (C) 2000, UAB "BBD SOFT" ( http://www.bbdsoft.com/ )
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
//  The author of this program may be contacted at developers@bbdsoft.com
//=============================================================================


#ifndef _PCI72XXCARD_
#define _PCI72XXCARD_

#ifndef _IOCARD_
   #include "iocard.hpp"
#endif

#ifndef _UNIIO_H
   #include "uniio.h"
#endif


//-----------------------------------------------------------------------------
namespace Uniio
{

class _Export_UNIIO PCI72xxCard : public IOCard
{

   typedef IOCard Inherited;

   public:
      PCI72xxCard ( const unsigned inCardId = 0 ); 
      virtual ~PCI72xxCard();

   protected:

      enum PCI72xxCardChannels
      {
         maxPCI7248Channels = 2,
         maxPCI7296Channels = 4,
         maxNumberOfPoints = 24,
         maxNumberOfPointsInBank = 8
      };

      virtual IOCard & updateInputs ( const IOTrack & inTrack
                                 , long * inImage
                                 , long & inTrackState
                                 );

      virtual IOCard & updateOutputs ( const IOTrack & inTrack
                                  , const long * const inImage
                                  , long & inTrackState
                                  );

      virtual IOCard & addTrack ( IOTrack & inTrack ) ;

     enum PCI72xxCardRegisters
     {
         regPortA       = 0x00 ,
         regPortB       = 0x01 ,
         regPortC       = 0x02 ,
         regControl     = 0x03 ,
         numberOfRegisters = 0x04 
     };

     enum PCI72xxCardConstants
     {
         TMR_OFFSET     = 0x10
        , TIMER0         = 0x10
        , TIMER1         = 0x11
        , TIMER2         = 0x12
        , TMR_MODE       = 0x13
        , INT_CTRL       = 0x20
        , CLR_IRQ        = 0x30
     };

     enum PCI72xxCardPorts
     {
         portA       = 0x00 ,
         portB       = 0x01 ,
         portC       = 0x02 ,
         maxBanks    = 0x03
     };

  private:

     unsigned short theInitFlag;
     unsigned long theLcrBase;
     unsigned long theBaseAddr;
     unsigned char theIrqNo;
     unsigned maxChannels;
     unsigned char * theMask;
};

};
#endif // _PCI72xxCard_
