//=============================================================================
//  Unified IO - C++ interface for industrial IO cards
//  Copyright (C) 2000, UAB "BBD SOFT" ( http://www.bbdsoft.com/ )
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
//  The author of this program may be contacted at developers@bbdsoft.com
//=============================================================================


#ifndef _PCIBUS_
#define _PCIBUS_

#ifndef _VECTOR_
   #include <vector>
#endif

#ifndef _UNIIO_H
   #include "uniio.h"
#endif

#ifndef _PCIDEV_
   #include "pcidev.hpp"
#endif

//-----------------------------------------------------------------------------
namespace Uniio
{

class PCIConfig;

class _Export_UNIIO PCIBus
{

   friend class PCIConfig;

public:

   static PCIBus & pciBus ();

   ~PCIBus ();

   bool exists() const;

   // returns NULL if device is not present in the system
   const PCIDeviceInfo * getDeviceInfo ( const unsigned short inVendorId
                         , const unsigned short inDeviceId
                         , const unsigned inCardNumber = 0 // if there is more than one card in the system 
                         ) const;
protected:


private:

   // disable copy constructor and assignment operator
   PCIBus (const PCIBus & );
   PCIBus & operator = (const PCIBus &);

   // the only instance of this class is created by function pcibus()
   PCIBus ();

   typedef std::vector<PCIDeviceInfo*> PCIDeviceList;

   void addDevice ( const unsigned char inBusNumber
                         , const unsigned char inSlotNumber
                         , void * inConfig
                         );

   void scanBus ();

   // pointer to the only instance of this class
   static PCIBus * thePCIBus;

   PCIDeviceList thePCIDeviceList;
   PCIConfig * theConfig;

}; // end PCIBus class declaration

}; // end namespace

#endif