//=============================================================================
//  Unified IO - C++ interface for industrial IO cards
//  Copyright (C) 2000, UAB "BBD SOFT" ( http://www.bbdsoft.com/ )
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
//  The author of this program may be contacted at developers@bbdsoft.com
//=============================================================================


#ifndef _BTRACE_
   #include "btrace.hpp"
#endif

#ifndef _PCIBUS_
   #include "pcibus.hpp"
#endif

#ifndef _PCICFG2_
   #include "pcicfg2.hpp"
#endif

#ifndef _PCICFGSL_
   #include "pcicfgsl.hpp"
#endif

#ifndef _STDEXCEPT_
   #include <stdexcept>
#endif

#include <cstddef>

using namespace Uniio;
using namespace std;

PCIBus * PCIBus::thePCIBus = NULL;


//-----------------------------------------------------------------------------
PCIBus & PCIBus::pciBus ()
{
BFUNCTRACE_DEVELOP ();

//If it is first call to the function - create PCI bus object
if (!thePCIBus)
{
   thePCIBus = new PCIBus;
   thePCIBus -> scanBus();
} // endif

return * thePCIBus;
} // end PCIBus::pciBus()


//-----------------------------------------------------------------------------
PCIBus::PCIBus ()
   : theConfig (NULL)
{
BFUNCTRACE_DEVELOP ();
} // end constructor


//-----------------------------------------------------------------------------
PCIBus::~PCIBus ()
{
BFUNCTRACE_DEVELOP ();
delete theConfig;
while (thePCIDeviceList.size())
{
   PCIDeviceList::iterator i = thePCIDeviceList.end();
   i--;
   delete (*i);
   thePCIDeviceList.pop_back();
}
} // end destructor


//-----------------------------------------------------------------------------
const PCIDeviceInfo * PCIBus::getDeviceInfo ( const unsigned short inVendorId
                           , const unsigned short inDeviceId
                           , const unsigned inCardNumber
                           ) const
{
BFUNCTRACE_DEVELOP();
unsigned aCardNumber = 0;

for ( PCIDeviceList::const_iterator i = thePCIDeviceList.begin()
    ; i != thePCIDeviceList.end()
    ; i++
    )
{
   const PCIDeviceInfo * anInfo = (*i);
   if (  anInfo -> vendorId() == inVendorId
      && anInfo -> deviceId() == inDeviceId
      )
   {
      if (aCardNumber == inCardNumber)
         return anInfo;
      else
         aCardNumber++;
   }
}
return NULL;
} // end PCIBus::isCardPresent (..)

//-----------------------------------------------------------------------------
void PCIBus::addDevice ( const unsigned char inBus
                         , const unsigned char inSlot
                         , void * inConfig
                         )
{
BFUNCTRACE_DEVELOP ();
thePCIDeviceList.push_back ( new PCIDeviceInfo (inBus, inSlot, inConfig) );
} // end 


//-----------------------------------------------------------------------------
void PCIBus::scanBus ()
{
BFUNCTRACE_DEVELOP ();
theConfig = PCIConfigSelector::createPCIConfig();
if (theConfig)
   theConfig -> scanBus ();
} // end 

//-----------------------------------------------------------------------------
bool PCIBus::exists () const
{
BFUNCTRACE_DEVELOP ();
return (theConfig != NULL);
} // end 
