//=============================================================================
//  Unified IO - C++ interface for industrial IO cards
//  Copyright (C) 2000, UAB "BBD SOFT" ( http://www.bbdsoft.com/ )
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
//  The author of this program may be contacted at developers@bbdsoft.com
//=============================================================================


#ifndef _BTRACE_
   #include "btrace.hpp"
#endif

#ifndef _PCICFG1_
   #include "pcicfg1.hpp"
#endif

using namespace Uniio;

// Command word for Config1 access
#define CONFIG_CMD(inBus, inSlot, inOffset)   (0x80000000L | ((inBus & 0xff) << 16) | (inSlot << 8) | (inOffset & ~3))

//-----------------------------------------------------------------------------
PCIConfig1::PCIConfig1 ()
   : theCommandPort (0xCF8)
{
BFUNCTRACE_DEVELOP ();
} // end constructor


//-----------------------------------------------------------------------------
PCIConfig1::~PCIConfig1 ()
{
BFUNCTRACE_DEVELOP ();
} // end destructor


//-----------------------------------------------------------------------------
bool PCIConfig1::exists() const
{
BFUNCTRACE_DEVELOP ();
   IOPort aPort (0xCFB);
   aPort.write ((unsigned char)0x01);

   IOPort aPort2 (0xCF8);
   unsigned long tmp = aPort2.readLong ();
   aPort2.write (0x80000000L);
   if (aPort2.readLong() == 0x80000000)
   {
      aPort2.write (tmp);
      return true;
   }
   return false;
}

//-----------------------------------------------------------------------------
unsigned char
PCIConfig1::readChar ( const unsigned char inBus
                     , const unsigned char inSlot
                     , const unsigned char inOffset ) const
{
BFUNCTRACE_DEVELOP ();
   theCommandPort.write (CONFIG_CMD(inBus,inSlot,inOffset));
   IOPort aDataPort (0xCFC + (inOffset&3));
   return aDataPort.readChar();
}

//-----------------------------------------------------------------------------
unsigned short
PCIConfig1::readShort ( const unsigned char inBus
                      , const unsigned char inSlot
                      , const unsigned char inOffset) const
{
BFUNCTRACE_DEVELOP ();
   theCommandPort.write (CONFIG_CMD(inBus,inSlot,inOffset));
   IOPort aDataPort (0xCFC + (inOffset&3));
   return aDataPort.readShort();
}

//-----------------------------------------------------------------------------
unsigned long
PCIConfig1::readLong ( const unsigned char inBus
                     , const unsigned char inSlot
                     , const unsigned char inOffset) const
{
BFUNCTRACE_DEVELOP ();
   theCommandPort.write (CONFIG_CMD(inBus,inSlot,inOffset));
   IOPort aDataPort (0xCFC+ (inOffset&3));
   return aDataPort.readLong();
}

//-----------------------------------------------------------------------------
void
PCIConfig1::write ( const unsigned char inBus
                  , const unsigned char inSlot
                  , const unsigned char inOffset
                  , const unsigned char inValue)
{
BFUNCTRACE_DEVELOP ();
   theCommandPort.write (CONFIG_CMD(inBus,inSlot,inOffset));
   IOPort aDataPort (0xCFC+ (inOffset&3));
   aDataPort.write (inValue);
}

//-----------------------------------------------------------------------------
void
PCIConfig1::write ( const unsigned char inBus
                  , const unsigned char inSlot
                  , const unsigned char inOffset
                  , const unsigned short inValue)
{
BFUNCTRACE_DEVELOP ();
   theCommandPort.write (CONFIG_CMD(inBus,inSlot,inOffset));
   IOPort aDataPort (0xCFC+ (inOffset&3));
   aDataPort.write (inValue);
}

//-----------------------------------------------------------------------------
void
PCIConfig1::write ( const unsigned char inBus
                  , const unsigned char inSlot
                  , const unsigned char inOffset
                  , const unsigned long inValue)
{
BFUNCTRACE_DEVELOP ();
   theCommandPort.write (CONFIG_CMD(inBus,inSlot,inOffset));
   IOPort aDataPort (0xCFC+ (inOffset&3));
   aDataPort.write (inValue);
}
