//=============================================================================
//  Unified IO - C++ interface for industrial IO cards
//  Copyright (C) 2000, UAB "BBD SOFT" ( http://www.bbdsoft.com/ )
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
//  The author of this program may be contacted at developers@bbdsoft.com
//=============================================================================


#ifndef _BTRACE_
   #include "btrace.hpp"
#endif

#ifndef _PCICFG2_
   #include "pcicfg2.hpp"
#endif

#include <stdexcept>

using namespace Uniio;
using namespace std;


// Functions for accessing PCI configuration space with type 2 accesses
#define IOADDR(inSlot, inOffset)   ((0xC000 | ((inSlot & 0x78) << 5)) + inOffset)
#define FUNC(inSlot)            ((unsigned char)(((inSlot & 7) << 1) | 0xf0))

//-----------------------------------------------------------------------------
PCIConfig2::PCIConfig2 ()
   : theFunctionPort (0xCF8)
   , theBusPort (0xCFA)
{
BFUNCTRACE_DEVELOP ();
} // end constructor


//-----------------------------------------------------------------------------
PCIConfig2::~PCIConfig2 ()
{
BFUNCTRACE_DEVELOP ();
} // end destructor


//-----------------------------------------------------------------------------
bool PCIConfig2::exists() const
{
BFUNCTRACE_DEVELOP ();
  // This is ugly and tends to produce false positives. Beware.
   IOPort aPort (0xCFB);
   IOPort aPort1 (0xCFA);
   IOPort aPort2 (0xCF8);
   aPort.write ((unsigned char)0x00);
   aPort1.write ((unsigned char)0x00);
   aPort2.write ((unsigned char)0x00);

   if (  aPort2.readChar() == 0x00
      && aPort1.readChar() == 0x00
      )
      return true;

   return false;
}

//-----------------------------------------------------------------------------
unsigned char
PCIConfig2::readChar ( const unsigned char inBus
                     , const unsigned char inSlot
                     , const unsigned char inOffset ) const
{
BFUNCTRACE_DEVELOP ();

   if (inSlot & 0x80)
      throw runtime_error ("PCI config2 supports only 16 devices per bus");

   theFunctionPort.write (FUNC(inSlot));
   theBusPort.write (inBus);

   IOPort aDataPort (IOADDR(inSlot,inOffset));
   unsigned char aValue = aDataPort.readChar();
   theFunctionPort.write ((unsigned char)0);
   return aValue;
}

//-----------------------------------------------------------------------------
unsigned short
PCIConfig2::readShort ( const unsigned char inBus
                      , const unsigned char inSlot
                      , const unsigned char inOffset) const
{
BFUNCTRACE_DEVELOP ();
   if (inSlot & 0x80)
      throw runtime_error ("PCI config2 supports only 16 devices per bus");

   theFunctionPort.write (FUNC(inSlot));
   theBusPort.write (inBus);

   IOPort aDataPort (IOADDR(inSlot,inOffset));
   unsigned short aValue = aDataPort.readShort();
   theFunctionPort.write ((unsigned char)0);
   return aValue;
}

//-----------------------------------------------------------------------------
unsigned long
PCIConfig2::readLong ( const unsigned char inBus
                     , const unsigned char inSlot
                     , const unsigned char inOffset) const
{
BFUNCTRACE_DEVELOP ();
   if (inSlot & 0x80)
      throw runtime_error ("PCI config2 supports only 16 devices per bus");

   theFunctionPort.write (FUNC(inSlot));
   theBusPort.write (inBus);

   IOPort aDataPort (IOADDR(inSlot,inOffset));
   unsigned long aValue = aDataPort.readLong();
   theFunctionPort.write ((unsigned char)0);
   return aValue;
}

//-----------------------------------------------------------------------------
void
PCIConfig2::write ( const unsigned char inBus
                  , const unsigned char inSlot
                  , const unsigned char inOffset
                  , const unsigned char inValue)
{
BFUNCTRACE_DEVELOP ();
   if (inSlot & 0x80)
      throw runtime_error ("PCI config2 supports only 16 devices per bus");

   theFunctionPort.write (FUNC(inSlot));
   theBusPort.write (inBus);

   IOPort aDataPort (IOADDR(inSlot,inOffset));
   aDataPort.write(inValue);
   theFunctionPort.write ((unsigned char)0);
}

//-----------------------------------------------------------------------------
void
PCIConfig2::write ( const unsigned char inBus
                  , const unsigned char inSlot
                  , const unsigned char inOffset
                  , const unsigned short inValue)
{
BFUNCTRACE_DEVELOP ();
   if (inSlot & 0x80)
      throw runtime_error ("PCI config2 supports only 16 devices per bus");

   theFunctionPort.write (FUNC(inSlot));
   theBusPort.write (inBus);

   IOPort aDataPort (IOADDR(inSlot,inOffset));
   aDataPort.write(inValue);
   theFunctionPort.write ((unsigned char)0);
}

//-----------------------------------------------------------------------------
void
PCIConfig2::write ( const unsigned char inBus
                  , const unsigned char inSlot
                  , const unsigned char inOffset
                  , const unsigned long inValue)
{
BFUNCTRACE_DEVELOP ();
   if (inSlot & 0x80)
      throw runtime_error ("PCI config2 supports only 16 devices per bus");

   theFunctionPort.write (FUNC(inSlot));
   theBusPort.write (inBus);

   IOPort aDataPort (IOADDR(inSlot,inOffset));
   aDataPort.write(inValue);
   theFunctionPort.write ((unsigned char)0);
}
