//=============================================================================
//  Unified IO - C++ interface for industrial IO cards
//  Copyright (C) 2000, UAB "BBD SOFT" ( http://www.bbdsoft.com/ )
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
//  The author of this program may be contacted at developers@bbdsoft.com
//=============================================================================


#ifndef _BTRACE_
   #include "btrace.hpp"
#endif

#ifndef _PCICFGDR_
   #include "pcicfgdr.hpp"
#endif

#include <cstddef>
#include <stdexcept>
#include "header.h"

using namespace Uniio;
using namespace std;

//-----------------------------------------------------------------------------
PCIConfigDirect::PCIConfigDirect ()
   : PCIConfig ()
{
BFUNCTRACE_DEVELOP ();
} // end constructor


//-----------------------------------------------------------------------------
PCIConfigDirect::~PCIConfigDirect ()
{
BFUNCTRACE_DEVELOP ();
} // end destructor


//-----------------------------------------------------------------------------
void
PCIConfigDirect::scanBus ()
{
BFUNCTRACE_DEVELOP ();
   if (!exists())
       throw runtime_error ("PCI bus scanning by this Config type is not possible");
   scanBus (0);
} // end 

//-----------------------------------------------------------------------------
void
PCIConfigDirect::scanBus ( const unsigned char inBus )
{
BFUNCTRACE_DEVELOP ();

  unsigned multi = 0;

  for(unsigned dev=0; dev<32; dev++)
  {
      multi = 0;
      for (unsigned func=0; func<8; func++)
      {
         unsigned char aSlot = PCI_DEVFN(dev, func);
         unsigned long aVendorId = 0;
         aVendorId = readLong ( inBus, aSlot, PCI_VENDOR_ID );

         // some broken boards return 0 if a slot is empty:
         if (  aVendorId == 0xffffffff
            || aVendorId == 0x00000000)
         {
            break;
         }

         unsigned long * aBuffer = new unsigned long [64];
         aBuffer[0] = aVendorId;
         for (unsigned i =1; i<64; i++)
         {
             aBuffer[i] = readLong ( inBus, aSlot, i*4 );
         }

         // insert this device 
         addDevice (inBus, aSlot, aBuffer);

         unsigned char ht = ((char*)aBuffer) [PCI_HEADER_TYPE];
         if (!func)
            multi = ht & 0x80;
         ht &= 0x7f;

         switch (ht)
         {
            case PCI_HEADER_TYPE_NORMAL:
              break;
            case PCI_HEADER_TYPE_BRIDGE:
            case PCI_HEADER_TYPE_CARDBUS:
               scanBus( ((char*)aBuffer) [ PCI_SECONDARY_BUS ] );
               break;
            default:
               throw runtime_error ("Unknown header type");
          }
          if (!multi)
            break;
        } // end for func
    } // end for device

} // end 

