/*
 * This software is in the public domain.
 * You may use it as you see fit with the understanding that
 * we make no guarantees of its fitness for any particular purpose.
 *
 * Author: XESS Corporation
 *         www.xess.com
 */

//
// The AT17Port object specializes the PPort object so that an XS40 Board serial EEPROM can
// be downloaded through the parallel port.
//

// $Log: at17prt.h $
// Revision 1.1  2001/12/30 15:34:59  DEVB
// Initial revision
//
// Revision 1.1  2001/12/30 15:26:33  DEVB
// Handles downloads to AT17C serial EEPROM through parallel port
//


#ifndef AT17Port_H
#define AT17Port_H

#include "Progress.h"
#include "HexRecrd.h"
#include "pport.h"


class AT17Port : public PPort
{
	public:

	/* begin AT17Port methods */

	/* method: AT17Port */
	// Create a serial EEPROM download port
	 AT17Port(void);

	/* method: AT17Port */
	// Create a serial EEPROM download port
	 AT17Port(
			XSError* e,	// pointer to error reporting object
			unsigned int portNum,	// parallel port number
			unsigned int invMask,	// inversion mask for the parallel port
			unsigned int pos_clk,	// bit position in parallel port of EEPROM clock pin
			unsigned int pos_eece,	// bit position in parallel port of EEPROM chip-enable
			unsigned int pos_eeoe,	// bit position in parallel port of EEPROM output-enable
			unsigned int pos_din		// bit position in parallel port of EEPROM serial data input
			);

	/* method: Setup */
	// Setup a serial EEPROM download port
	int Setup(
			XSError* e,	// pointer to error reporting object
			unsigned int portNum,	// parallel port number
			unsigned int invMask,	// inversion mask for the parallel port
			unsigned int pos_clk,	// bit position in parallel port of EEPROM clock pin
			unsigned int pos_eece,	// bit position in parallel port of EEPROM chip-enable
			unsigned int pos_eeoe,	// bit position in parallel port of EEPROM output-enable
			unsigned int pos_din		// bit position in parallel port of EEPROM serial data input
			);

	/* method: ProgramEEPROM */
	// download the serial EEPROM with the contents of a BIT file
	bool ProgramEEPROM(string& bitfileName);

	/* method: ProgramEEPROM */
	// program a serial EEPROM with a bitstream
	bool ProgramEEPROM(istream& is);

	/* method: SendEEPROMByte */
	// Send a byte into an Atmel EEPROM
	void SendEEPROMByte(unsigned char byte);

	/* method: ProgramEEPROMPage */
	// Program a page of an Atmel EEPROM
	void ProgramEEPROMPage(
			unsigned char* buf,	// buffer with data bytes
			unsigned int pageAddr,	// address where data will be stored
			unsigned int len		// number of bytes in buffer
			);

	/* end AT17Port methods */


	protected:

	Progress *progressGauge;	// indicates progress of operations

	
	private:

	unsigned int posCLK;	// bit position in parallel port of EEPROM clock pin
	unsigned int posEECE;	// bit position in parallel port of EEPROM chip-enable
	unsigned int posEEOE;	// bit position in parallel port of EEPROM output-enable
	unsigned int posDIN;	// bit position in parallel port of EEPROM serial data input
};

/* begin AT17Port functions */

/* end AT17Port functions */

#endif
