/*
 * This software is in the public domain.
 * You may use it as you see fit with the understanding that
 * we make no guarantees of its fitness for any particular purpose.
 *
 * Author: XESS Corporation
 *         www.xess.com
 */

//
// The AT49FPort object specializes the PPort object so that an XSA Board AT49F Flash can
// be downloaded and uploaded through the parallel port.
//

// $Log: at49fprt.h $
// Revision 1.1  2001/12/30 15:36:29  DEVB
// Initial revision
//
// Revision 1.1  2001/12/30 15:29:36  DEVB
// <Empty change comment>
//


#ifndef AT49FPort_H
#define AT49FPort_H

#include "FlashPrt.h"


class AT49FPort : public FlashPort
{
	public:

	/* begin AT49FPort methods */

	/* method: AT49FPort */
	// Create a Flash upload/download port
	 AT49FPort(void);

	/* method: AT49FPort */
	// Create a Flash upload/download port
	 AT49FPort(XSError* e, // pointer to error reporting object
		unsigned int portNum, // parallel port number
		unsigned int invMask, // inversion mask for the parallel port
		unsigned int pos_reset, // bit position in parallel port of Flash RESET pin
		unsigned int pos_clk, // bit position in parallel port of Flash CLK pin
		unsigned int pos_dolsb, // bit position in parallel port of LSB of Flash data-out pin
		unsigned int pos_domsb, // bit position in parallel port of MSB of Flash data-out pin
		unsigned int pos_dilsb, // bit position in parallel port of LSB of Flash data-in pin
		unsigned int pos_dimsb, // bit position in parallel port of MSB of Flash data-in pin
		unsigned int pos_stlsb, // bit position in parallel port of LSB of Flash status pin
		unsigned int pos_stmsb) // bit position in parallel port of MSB of Flash status pin
		;

	/* method: ProgramFlash */
	// Program a byte in the Flash
	bool ProgramFlash(unsigned int address, unsigned int data);

	/* method: EraseFlash */
	// erase the entire Flash RAM
	bool EraseFlash(void);

	/* method: EraseFlashBlock */
	// erase the contents of a Flash block
	bool EraseFlashBlock(unsigned int blockIndex);

	/* method: ResetFlash */
	// reset the Flash so data can be read from it
	bool ResetFlash(void);

	/* end AT49FPort methods */

};

/* begin AT49FPort functions */

/* end AT49FPort functions */

#endif
