/*
 * This software is in the public domain.
 * You may use it as you see fit with the understanding that
 * we make no guarantees of its fitness for any particular purpose.
 *
 * Author: XESS Corporation
 *         www.xess.com
 */

//
// Bitstream objects just store strings of boolean values and
// allow certain operations to be performed on them
//

// $Log: bitstrm.h $
// Revision 1.3  2001/12/30 16:04:04  DEVB
// Added method to convert Bitstream to a binary string
//
// Revision 1.2  1999/03/30 18:26:26  DEVB
// Released source into the public domain
//
// Revision 1.1  1999/03/29 22:56:05  DEVB
// Initial revision
//

#ifndef Bitstream_H
#define Bitstream_H


#include <iostream.h>
#include <assert.h>
#include <stdarg.h>
#include <string>
using std::string;


class Bitstream
{
	public:

	/* begin Bitstream methods */

	/* method: Bitstream */
	// allocates a bitstream containing at least n bits.
	 Bitstream(unsigned int n);

	/* method: ~Bitstream */
	// frees the storage used by a bitstream.
	 ~Bitstream(void);

	/* method: GetLength */
	// returns the number of bits in the bitstream
	unsigned int GetLength(void) const;

	/* method: operator== */
	// Compares two bitstreams.  Returns true if they match, false otherwise.
	bool operator==(Bitstream& b2) const;

	/* method: Subcompare */
	// compares the end of 1st bitstream against a subfield of 2nd bitstream
	bool Subcompare(unsigned int pos, // position in 1st bitstream
		Bitstream& b2, // 2nd bitstream
		unsigned int b2Pos) const // position in 2nd bitstream
		;

	/* method: operator= */
	// copies contents of one bitstream into another (also adjusts size).
	Bitstream& operator=(Bitstream& b2);

	/* method: Copy */
	// makes a new copy of a bitstream
	Bitstream* Copy(void);

	/* method: operator[] */
	// returns the value of a bit from a bitstream.
	unsigned int operator[](unsigned int bitIndex) const;

	/* method: Clear */
	// clear a bitstream
	void Clear(void);

	/* method: SetBit */
	// sets the value of a bit in a bitstream
	void SetBit(unsigned int bitIndex, // position to set
		unsigned int val // value to set bit
		);

	/* method: SetBits */
	// Sets the value of multiple bits in a bitstream.  End the string of bit values
	// with a number that is neither 0 or 1.
	void SetBits(unsigned int bitIndex, // position to start changing bits
		int firstBit, ... // list of bit values
		);

	/* method: operator^ */
	// XOR two bitstreams and return the resulting bitstream.
	Bitstream& operator^(Bitstream& b2) const;

	/* method: operator+ */
	// concatenate two bitstreams to create a third bitstream
	Bitstream& operator+(Bitstream& b2) const;

	/* method: printBits */
	// prints a bitstream to an output stream
	ostream& PrintBits(ostream& os) const;

	/* method: ToString */
	// converts a bitstream to a string of 1s and 0s
	string ToString(void) const;

	private:

	unsigned int numBits;	// number of bits in bitstream
	unsigned long* bits;	// storage for bitstream

	/* end Bitstream methods */
};

/* begin Bitstream functions */

/* function: operator<< */
// helper function which prints a bitstream to an output stream
ostream& operator<<(ostream& os, Bitstream& b);

/* end Bitstream functions */

#endif
