/*
 * This software is in the public domain.
 * You may use it as you see fit with the understanding that
 * we make no guarantees of its fitness for any particular purpose.
 *
 * Author: XESS Corporation
 *         www.xess.com
 */

//
// ConfigPort objects add FPGA configuration capabilities to a parallel port object.
// A ConfigPort can control the CCLK, /PROGRAM, and DIN pins and read the
// DONE pin.
//

// $Log: cnfgport.cpp $
// Revision 1.1  2001/12/30 15:37:53  DEVB
// Initial revision
//



#include <string>
using std::string;



#include "CnfgPort.h"



// begin methods and functions



// Create a config controller port
CnfgPort::CnfgPort(void)
{
	;
}

// Create a config controller port
CnfgPort::CnfgPort(XSError* e,	// pointer to error reporting object
				   unsigned int portNum,	// parallel port number
				   unsigned int invMask,	// inversion mask for the parallel port
				   unsigned int pos_cclk,	// bit position in parallel port of CCLK pin
				   unsigned int pos_prog,	// bit position in parallel port of PROGRAM pin
				   unsigned int pos_din,	// bit position in parallel port of DIN pin
				   unsigned int pos_done	// bit position in parallel port of DONE pin
				   )
{
	Setup(e,portNum,invMask,pos_cclk,pos_prog,pos_din,pos_done);
}



// Setup a config controller port
bool CnfgPort::Setup(XSError* e,	// pointer to error reporting object
				   unsigned int portNum,	// parallel port number
				   unsigned int invMask,	// inversion mask for the parallel port
				   unsigned int pos_cclk,	// bit position in parallel port of CCLK pin
				   unsigned int pos_prog,	// bit position in parallel port of PROGRAM pin
				   unsigned int pos_din,	// bit position in parallel port of DIN pin
				   unsigned int pos_done	// bit position in parallel port of DONE pin
				   )
{
	posCCLK = pos_cclk;
	posPROG = pos_prog;
	posDIN = pos_din;
	posDONE = pos_done;
	return PPort::Setup(e,portNum,invMask);	// return false if error occurs
}



// Set the value of the CCLK pin
void CnfgPort::SetCCLK(unsigned int b)
{
	Out(b,posCCLK,posCCLK);
}

// Get the value output on the CCLK pin
unsigned int CnfgPort::GetCCLK(void)
{
	return In(posCCLK,posCCLK);
}

// Toggle the CCLK pin twice (return it to its original level)
void CnfgPort::PulseCCLK(void)
{
	assert(GetCCLK()==0);  // quiescent state of CCLK should be zero
	SetCCLK(1);  // toggle TCK output
	SetCCLK(0);	// toggle it again
}



// Set the value of the PROGRAM pin
void CnfgPort::SetPROG(unsigned int b)
{
	Out(b,posPROG,posPROG);
}

// Get the value output on the PROGRAM pin
unsigned int CnfgPort::GetPROG(void)
{
	return In(posPROG,posPROG);
}

// Toggle the configuration initiation pin (return it to its original level)
void CnfgPort::PulsePROG(void)
{
	assert(GetPROG()==1);	// quiescent state of /PROGRAM should be logic high level
	SetPROG(0);				// toggle /PROGRAM output
	SetPROG(1);				// toggle it again
}


// Set the value of the DIN pin
void CnfgPort::SetDIN(unsigned int b)
{
	Out(b,posDIN,posDIN);
}

// Get the value output on the DIN pin
unsigned int CnfgPort::GetDIN(void)
{
	return In(posDIN,posDIN);
}



// Get the value of the DONE pin
unsigned int CnfgPort::GetDONE(void)
{
	return In(posDONE,posDONE);
}
