/*
 * This software is in the public domain.
 * You may use it as you see fit with the understanding that
 * we make no guarantees of its fitness for any particular purpose.
 *
 * Author: XESS Corporation
 *         www.xess.com
 */

//
// ConfigPort objects add FPGA configuration capabilities to a parallel port object.
// A ConfigPort can control the CCLK, /PROGRAM, and DIN pins and read the
// DONE pin.
//

// $Log: cnfgport.h $
// Revision 1.1  2001/12/30 15:37:53  DEVB
// Initial revision
//



#ifndef CnfgPort_H
#define CnfgPort_H

#include <assert.h>
#include <stdarg.h>

#include "pport.h"



class CnfgPort : public PPort
{
	public:

	/* begin CnfgPort methods */

	/* method: CnfgPort */
	// Create a config controller port
	CnfgPort(void);

	/* method: CnfgPort */
	// Create a config controller port
	CnfgPort(XSError* e, // pointer to error reporting object
		unsigned int portNum, // parallel port number
		unsigned int invMask, // inversion mask for the parallel port
		unsigned int pos_cclk, // bit position in parallel port of CCLK pin
		unsigned int pos_prog, // bit position in parallel port of PROGRAM pin
		unsigned int pos_din, // bit position in parallel port of DIN pin
		unsigned int pos_done); // bit position in parallel port of DONE pin

	/* method: Setup */
	// Setup a config controller port
	bool Setup(XSError* e, // pointer to error reporting object
		unsigned int portNum, // parallel port number
		unsigned int invMask, // inversion mask for the parallel port
		unsigned int pos_cclk, // bit position in parallel port of CCLK pin
		unsigned int pos_prog, // bit position in parallel port of PROGRAM pin
		unsigned int pos_din, // bit position in parallel port of DIN pin
		unsigned int pos_done); // bit position in parallel port of DONE pin

	/* method: SetCCLK */
	// Set the value of the CCLK pin
	void SetCCLK(unsigned int b);

	/* method: GetCCLK */
	// Get the value output on the CCLK pin
	unsigned int GetCCLK(void);

	/* method: PulseCCLK */
	// Toggle the CCLK pin twice (return it to its original level)
	void PulseCCLK(void);

	/* method: SetPROG */
	// Set the value of the PROGRAM pin
	void SetPROG(unsigned int b);

	/* method: GetPROG */
	// Get the value output on the PROGRAM pin
	unsigned int GetPROG(void);

	/* method: PulsePROG */
	// Toggle the configuration initiation pin (return it to its original level)
	void PulsePROG(void);

	/* method: SetDIN */
	// Set the value of the DIN pin
	void SetDIN(unsigned int b);

	/* method: GetDIN */
	// Get the value output on the DIN pin
	unsigned int GetDIN(void);

	/* method: GetDONE */
	// Get the value of the DONE pin
	unsigned int GetDONE(void);

	/* end CnfgPort methods */

	
	private:

	// These store the bit positions of the FPGA config. control lines
	unsigned int posCCLK;	// position of configuration clock pin
	unsigned int posPROG;	// position of configuration initiation pin 
	unsigned int posDIN;	// position of configuration data pin
	unsigned int posDONE;	// position of configuration done pin

};

/* begin CnfgPort functions */

/* end CnfgPort functions */

#endif
