/*
 * This software is in the public domain.
 * You may use it as you see fit with the understanding that
 * we make no guarantees of its fitness for any particular purpose.
 *
 * Author: XESS Corporation
 *         www.xess.com
 */

//
// The F28Port object specializes the PPort object so that an XSV Board 28F016 Flash can
// be downloaded and uploaded through the parallel port.
//

// $Log: f28port.h $
// Revision 1.1  2001/12/30 15:39:41  DEVB
// Initial revision
//


#ifndef F28Port_H
#define F28Port_H

#include "FlashPrt.h"


class F28Port : public FlashPort
{
	public:

	/* begin F28Port methods */

	/* method: F28Port */
	// Create a Flash upload/download port
	 F28Port(void);

	/* method: F28Port */
	// Create a Flash upload/download port
	 F28Port(XSError* e, // pointer to error reporting object
		unsigned int portNum, // parallel port number
		unsigned int invMask, // inversion mask for the parallel port
		unsigned int pos_reset, // bit position in parallel port of Flash RESET pin
		unsigned int pos_clk, // bit position in parallel port of Flash CLK pin
		unsigned int pos_dolsb, // bit position in parallel port of LSB of Flash data-out pin
		unsigned int pos_domsb, // bit position in parallel port of MSB of Flash data-out pin
		unsigned int pos_dilsb, // bit position in parallel port of LSB of Flash data-in pin
		unsigned int pos_dimsb, // bit position in parallel port of MSB of Flash data-in pin
		unsigned int pos_stlsb, // bit position in parallel port of LSB of Flash status pin
		unsigned int pos_stmsb) // bit position in parallel port of MSB of Flash status pin
		;

	/* method: ProgramFlash */
	// Program a byte in the Flash
	bool ProgramFlash(unsigned int address, unsigned int data);

	/* method: EraseFlash */
	// erase the entire Flash RAM
	bool EraseFlash(void);

	/* method: EraseFlashBlock */
	// erase the contents of a Flash block
	bool EraseFlashBlock(unsigned int blockIndex);

	/* method: ResetFlash */
	// reset the Flash so data can be read from it
	bool ResetFlash(void);

	/* method: ReadFlashID */
	// read the ID from the Flash
	bool ReadFlashID(unsigned int* id);

	/* method: ReadFlashStatus */
	// get the status of the Flash
	bool ReadFlashStatus(unsigned int* status);

	/* method: ClearFlashStatus */
	// clear any error bits in the Flash status register
	bool ClearFlashStatus(void);

	/* method: LockFlashBlock */
	// lock a Flash block so it can't be altered
	bool LockFlashBlock(unsigned int blockIndex);

	/* method: MasterLockFlash */
	// lock all the block lock bits so they cannot be altered
	bool MasterLockFlash(void);

	/* method: UnlockFlashBlocks */
	// unlock all the Flash blocks
	bool UnlockFlashBlocks(void);

	/* end F28Port methods */

};

/* begin F28Port functions */

/* end F28Port functions */

#endif
