/*
 * This software is in the public domain.
 * You may use it as you see fit with the understanding that
 * we make no guarantees of its fitness for any particular purpose.
 *
 * Author: XESS Corporation
 *         www.xess.com
 */

//
// Hex objects just store hexadecimal numbers with
// a set number of digits.
//

// $Log: hex.cpp $
// Revision 1.3  2001/12/30 16:05:48  DEVB
// Added assertion check and expanded buffer.
//
// Revision 1.2  1999/03/30 18:27:12  DEVB
// Released source into the public domain
//
// Revision 1.1  1999/03/29 21:51:22  DEVB
// Initial revision
//

#include <stdlib.h>
#include <iomanip.h>
#include <ctype.h>
#include <assert.h>

#include "hex.h"
#include "utils.h"



// begin methods and functions

// construct a hex number with a given number of hex digits
Hex::Hex(unsigned int l)
{
	SetLength(l);	// set number of hex digits in hex number
	SetHex(0);		// initialize hex number to 0
}

// set the number of hex digits in the hex number
void Hex::SetLength(unsigned int l)
{
	assert(sizeof(unsigned int)/4 >= l);
	length = l;
}

// return the number of hex digits in the hex number
unsigned int Hex::GetLength(void)
{
	return length;
}

// assign a value to the hex number
void Hex::SetHex(unsigned int h)
{
	hex = h;
}

// assign a value to the hex number
Hex& Hex::operator=(unsigned int h)
{
	hex = h;
	return *this;
}

// return the value of the hex number
unsigned int Hex::GetHex(void)
{
	return hex;
}

// helper function to convert ASCII '0'-'F' to number between 0 and 15, inclusive
unsigned int CharToHex(char c)
{
	if(c>='0' && c<='9')
		return c-'0';
	c = toupper(c);
	return c-'A'+10;
}

// helper function which gets a hex number from an input stream
istream& operator>> (istream& is, Hex& n)
{
	char buf[100];
	is >> setw(n.GetLength()+1) >> buf;
	n.SetHex(0);
	bool started = false;
	for(char *p=buf; *p!='\0'; p++)
	{
		if(isspace(*p))
		{
			if(started) break; // stop once numerals end
			else continue;     // keep looking for start of numerals
		}
		started = true;
		n.SetHex(n.GetHex()*16 + CharToHex(*p));
	}
	return is;
}

// helper function which outputs a hex number to an output stream
ostream& operator<< (ostream& os, Hex& n)
{
	os << n.GetHex();
	return os;
}
