/*
 * This software is in the public domain.
 * You may use it as you see fit with the understanding that
 * we make no guarantees of its fitness for any particular purpose.
 *
 * Author: XESS Corporation
 *         www.xess.com
 */

//
// The RAMPort object specializes the JTAGPort object so that an XS Board RAM can
// be downloaded and uploaded through the parallel port using JTAG commands.
//

// $Log: jramprt.h $
// Revision 1.1  2001/12/30 15:42:41  DEVB
// Initial revision
//

#ifndef JTAGRAMPort_H
#define JTAGRAMPort_H

#include "Progress.h"
#include "JTAGPort.h"
#include "XCBSDR.h"
#include "HexRecrd.h"

enum {XC9572Type=0, XC95108Type=1};

class JTAGRAMPort : public JTAGPort
{
	public:

	/* begin JTAGRAMPort methods */

	/* method: JTAGRAMPort */
	// Create a memory upload/download port
	JTAGRAMPort();

	/* method: JTAGRAMPort */
	// Create a memory upload/download port
	JTAGRAMPort(
		int xc95Type,			// type of XC9500 CPLD
		XSError* e,				// pointer to error reporting object
		unsigned int portNum,	// parallel port number
		unsigned int invMask,	// inversion mask for the parallel port
		unsigned int pos_tck,	// bit position in parallel port of TCK pin
		unsigned int pos_tms,	// bit position in parallel port of TMS pin
		unsigned int pos_tdi,	// bit position in parallel port of TDI pin
		unsigned int pos_tdo	// bit position in parallel port of TDO pin
		);

	/* method: Setup */
	// Setup a memory upload/download port
	bool Setup(
		int xc95Type,			// type of XC9500 CPLD
		XSError* e,				// pointer to error reporting object
		unsigned int portNum,	// parallel port number
		unsigned int invMask,	// inversion mask for the parallel port
		unsigned int pos_tck,	// bit position in parallel port of TCK pin
		unsigned int pos_tms,	// bit position in parallel port of TMS pin
		unsigned int pos_tdi,	// bit position in parallel port of TDI pin
		unsigned int pos_tdo	// bit position in parallel port of TDO pin
		);

	/* method: DownloadRAM */
	// download the XS Board RAM with the contents of a HEX file
	bool DownloadRAM(string& hexfileName);

	/* method: DownloadRAM */
	// download the XS Board RAM with the contents arriving through a stream
	bool DownloadRAM(istream& is);

	/* method: UploadRAM */
	bool UploadRAM(
		string& hexfileName, // dump uploaded data to this file
		const char* format,		// hex file format
		unsigned long loAddr, // start fetching data from this address
		unsigned long hiAddr // stop at this address
		);

	/* method: UploadRAM */
	// upload the XS Board RAM to a stream
	bool UploadRAM(
		ostream& os, // dump uploaded data to this stream
		const char* format,		// hex file format
		unsigned long loAddr, // start fetching data from this address
		unsigned long hiAddr // stop at this address
		);

	/* method: DownloadHexRecordToRAM */
	// load RAM data at the addresses given in a hex record
	bool DownloadHexRecordToRAM(HexRecord& hx);

	/* method: UploadHexRecordFromRAM */
	// get RAM data at the addresses given and store it in a hex record
	bool UploadHexRecordFromRAM(
		HexRecord& hx, // hex record to store data in
		unsigned long loAddr, // begin upload from this address
		unsigned long hiAddr // end upload at this address
		);

	/* end RAMPort methods */


	protected:

	Progress *progressGauge;	// indicates progress of operations

	
	private:

	Bitstream* bsirPtr;
	XCBSDR* bsdrPtr;
};

/* begin JTAGRAMPort functions */

/* end JTAGRAMPort functions */

#endif
