/*
 * This software is in the public domain.
 * You may use it as you see fit with the understanding that
 * we make no guarantees of its fitness for any particular purpose.
 *
 * Author: XESS Corporation
 *         www.xess.com
 */

//
// OscPort objects specialize a PPort to handle programming of
// a DS1075 programmable oscillator.
//

// $Log: oscport.h $
// Revision 1.1  2001/12/30 15:46:20  DEVB
// Initial revision
//

#ifndef OscPort_H
#define OscPort_H

#include <assert.h>
#include <stdarg.h>
#include "PPort.h"

class OscPort : PPort
{
	public:

	/* begin OscPort methods */

	/* method: OscPort */
	// Create a prog. oscillator controller port
	 OscPort(void);

	/* method: OscPort */
	// Create a prog. oscillator controller port
	 OscPort(XSError* e, // pointer to error reporting object
		unsigned int portNum, // parallel port number
		unsigned int invMask, // inversion mask for the parallel port
		unsigned int pos_osc); // bit position in parallel port of osc. pin

	/* method: Setup */
	// Setup a prog. oscillator controller port
	int Setup(XSError* e, // pointer to error reporting object
		unsigned int portNum, // parallel port number
		unsigned int invMask, // inversion mask for the parallel port
		unsigned int pos_osc); // bit position in parallel port of osc. pin

	/* method: ResetOsc */
	// total bit duration
	// reset the programmable oscillator
	void ResetOsc();

	/* method: SendOscBit */
	// send a bit of data to the programmable oscillator.
	void SendOscBit(unsigned char b);

	/* method: IssueOscCmd */
	// send a command to the programmable oscillator
	void IssueOscCmd(unsigned int cmd);

	/* method: SendOscData */
	// send a data word to the programmable oscillator
	void SendOscData(unsigned int data);

	/* method: SetOscFrequency */
	// set the DS1075 oscillator frequency
	bool SetOscFrequency(int div, bool extOscPresent);

	/* end OscPort methods */

	
	private:

	unsigned int posOsc; // position of prog. osc. clock pin

};

/* begin OscPort functions */

/* end OscPort functions */

#endif
