/*
 * This software is in the public domain.
 * You may use it as you see fit with the understanding that
 * we make no guarantees of its fitness for any particular purpose.
 *
 * Author: XESS Corporation
 *         www.xess.com
 */

//
// The Progress object reports the progress of an operation involving the XS Boards.
//

#include "progress.h"

#ifdef _WINDOWS
BEGIN_MESSAGE_MAP(Progress,CWnd)
//	ON_WM_PAINT()
END_MESSAGE_MAP()
#endif


#ifdef _WINDOWS
Progress::Progress(void) : CWnd()
#else
Progress::Progress(void)
#endif
{
	visible = false;	// progress indicator is not yet visible
}


#ifdef _WINDOWS
Progress::Progress(XSError* e, string& maintaskDesc, string& subtaskDesc, float lo, float hi) : CWnd()
#else
Progress::Progress(XSError* e, string& maintaskDesc, string& subtaskDesc, float lo, float hi)
#endif
{
	visible = false;	// progress indicator is not yet visible
	Setup(e,maintaskDesc,subtaskDesc,lo,hi);
}


Progress::~Progress(void)
{
#ifdef _WINDOWS
#endif
	cerr << "\n";	// go to new line once progress indicator dies
}


bool Progress::Setup(XSError* e, string& maintaskDesc, string& subtaskDesc, float lo, float hi)
{
	if(!visible)
	{ // only allow setup when the progress indicator has not yet been made visible
		errorChannel = e;
		maintaskDescription = maintaskDesc;
		subtaskDescription = subtaskDesc;
		loBound = lo;
		hiBound = hi;
		percentDone = 0;
		return true;	// setup successful
	}
	else
		return false;	// setup failed because progress indicator is already visible
}


void Progress::Report(float x)
{
	if(!visible)
	{ // progress indicator is not visible, so make it visible
		cerr << "\n" << maintaskDescription.data() << ":\n";
		cerr << subtaskDescription.data() << ":";
#ifdef _WINDOWS
		CString strWndClass = AfxRegisterWndClass(
			0,
			::LoadCursor(NULL,IDC_ARROW),
			(HBRUSH)(COLOR_3DFACE+1),
			0);
		
		CWnd* mainWnd = AfxGetMainWnd();
		CRect mainRect, progRect(0,0,250,80);
		mainWnd->GetWindowRect(&mainRect);
		progRect.OffsetRect(mainRect.TopLeft());
		progRect.OffsetRect(0,mainRect.Height()+20);
		CreateEx(0,strWndClass,maintaskDescription.data(),WS_POPUP|WS_CAPTION|WS_VISIBLE,
			progRect,AfxGetMainWnd(),0,NULL);
		description.Create(subtaskDescription.data(),WS_CHILD | WS_VISIBLE | SS_LEFT,
			CRect(10,10,235,25),this,0);

		CFont fntArial;
		fntArial.CreatePointFont(80, "MS Sans Serif");
		description.SetFont(&fntArial);
		description.RedrawWindow();

		ctlProgress.Create(WS_CHILD | WS_VISIBLE | WS_BORDER, CRect(10,30,235,45), this, 0);
		ctlProgress.SetPos(0);
		UpdateWindow();
#endif
		visible = true;	// progress indicator is now visible
	}
	
	percentDone = (x-loBound)/(hiBound-loBound) * 100;
	char msg1[20],msg2[20];
	sprintf(msg1,"%d%%",(int)percentDone);
	sprintf(msg2,"%4.4s\010\010\010\010",msg1);
	cerr << msg2;
#ifdef _WINDOWS
	ctlProgress.SetPos(percentDone);
	UpdateWindow();
#endif
}


#ifdef _WINDOWS
void Progress::OnPaint()
{
	ctlProgress.SetPos(percentDone);
}
#endif