/*
 * This software is in the public domain.
 * You may use it as you see fit with the understanding that
 * we make no guarantees of its fitness for any particular purpose.
 *
 * Author: XESS Corporation
 *         www.xess.com
 */

//
// The RAMPort object specializes the PPort object so that an XS Board RAM can
// be downloaded and uploaded through the parallel port.
//

// $Log: ramport.h $
// Revision 1.1  2001/12/30 15:45:21  DEVB
// Initial revision
//

#ifndef MemPort_H
#define MemPort_H

#include "Progress.h"
#include "PPort.h"
#include "HexRecrd.h"

class RAMPort : public PPort
{
	public:

	/* begin RAMPort methods */

	/* method: RAMPort */
	// Create a memory upload/download port
	 RAMPort();

	/* method: RAMPort */
	// Create a memory upload/download port
	 RAMPort(XSError* e, // pointer to error reporting object
		unsigned int portNum, // parallel port number
		unsigned int invMask, // inversion mask for the parallel port
		unsigned int pos_reset, // bit position in parallel port of RAM RESET pin
		unsigned int pos_clk, // bit position in parallel port of RAM CLK pin
		unsigned int pos_dolsb, // bit position in parallel port of LSB of RAM data-out pin
		unsigned int pos_domsb, // bit position in parallel port of MSB of RAM data-out pin
		unsigned int pos_dilsb, // bit position in parallel port of LSB of RAM data-in pin
		unsigned int pos_dimsb, // bit position in parallel port of MSB of RAM data-in pin
		unsigned int pos_stlsb, // bit position in parallel port of LSB of RAM status pin
		unsigned int pos_stmsb, // bit position in parallel port of MSB of RAM status pin
		unsigned int dWidth,	// width of data word
		unsigned int aWidth		// width of address
		);

	/* method: Setup */
	// Setup a memory upload/download port
	int Setup(XSError* e, // pointer to error reporting object
		unsigned int portNum, // parallel port number
		unsigned int invMask, // inversion mask for the parallel port
		unsigned int pos_reset, // bit position in parallel port of RAM RESET pin
		unsigned int pos_clk, // bit position in parallel port of RAM CLK pin
		unsigned int pos_dolsb, // bit position in parallel port of LSB of RAM data-out pin
		unsigned int pos_domsb, // bit position in parallel port of MSB of RAM data-out pin
		unsigned int pos_dilsb, // bit position in parallel port of LSB of RAM data-in pin
		unsigned int pos_dimsb, // bit position in parallel port of MSB of RAM data-in pin
		unsigned int pos_stlsb, // bit position in parallel port of LSB of RAM status pin
		unsigned int pos_stmsb, // bit position in parallel port of MSB of RAM status pin
		unsigned int dWidth,	// width of data word
		unsigned int aWidth		// width of address
		);

	/* method: DownloadRAM */
	// download the XS Board RAM with the contents of a HEX file
	bool DownloadRAM(string& hexfileName);

	/* method: DownloadRAM */
	// download the XS Board RAM with the contents arriving through a stream
	bool DownloadRAM(istream& is);

	/* method: UploadRAM */
	bool UploadRAM(
		string& hexfileName, // dump uploaded data to this file
		const char* format,		// hex file format
		unsigned long loAddr, // start fetching data from this address
		unsigned long hiAddr // stop at this address
		);

	/* method: UploadRAM */
	// upload the XS Board RAM to a stream
	bool UploadRAM(
		ostream& os, // dump uploaded data to this stream
		const char* format,		// hex file format
		unsigned long loAddr, // start fetching data from this address
		unsigned long hiAddr // stop at this address
		);

	/* method: DownloadHexRecordToRAM */
	// load RAM data at the addresses given in a hex record
	bool DownloadHexRecordToRAM(HexRecord& hx);

	/* method: UploadHexRecordFromRAM */
	// get RAM data at the addresses given and store it in a hex record
	bool UploadHexRecordFromRAM(
		HexRecord& hx, // hex record to store data in
		unsigned long loAddr, // begin upload from this address
		unsigned long hiAddr // end upload at this address
		);

	/* end RAMPort methods */


	protected:

	Progress *progressGauge;	// indicates progress of operations

	
	private:

	unsigned int posRESET; // bit position in parallel port of RAM RESET pin
	unsigned int posCLK;   // bit position in parallel port of RAM CLK pin
	unsigned int posDOLSB; // bit position in parallel port of LSB of RAM data-out pin
	unsigned int posDOMSB; // bit position in parallel port of MSB of RAM data-out pin
	unsigned int posDILSB; // bit position in parallel port of LSB of RAM data-in pin
	unsigned int posDIMSB; // bit position in parallel port of MSB of RAM data-in pin
	unsigned int posSTLSB; // bit position in parallel port of LSB of RAM status pin
	unsigned int posSTMSB; // bit position in parallel port of MSB of RAM status pin
	unsigned int dataWidth;	// width of RAM data
	unsigned int addrWidth;	// width of RAM address
};

/* begin RAMPort functions */

/* end RAMPort functions */

#endif
