/*
 * This software is in the public domain.
 * You may use it as you see fit with the understanding that
 * we make no guarantees of its fitness for any particular purpose.
 *
 * Author: XESS Corporation
 *         www.xess.com
 */

#ifndef utils_H
#define utils_H

#include <string>
using std::string;

const unsigned int MILLISECONDS = 1000;
const unsigned int MICROSECONDS = 1;

enum{PROMPT_OK,PROMPT_OKCANCEL};
enum{RESPONSE_CONTINUE,RESPONSE_CANCEL};

typedef struct brdModel_struct
{
	char* brdModel;					// XS Board model identifier
	char* chipType;					// identifier for main programmable chip on board
	char* dwnldIntfcBitstreamFile;	// standard interface for downloading/uploading bitstreams and RAM data
	char* ramBitstreamFile;			// RAM upload/download interface bistream file
	char* oscBitstreamFile;			// programmable oscillator interface bitstream file
	char* eraseBitstreamFile;		// erase file for interface CPLD on board
	char* flashBitstreamFile;		// Flash/SEEPROM upload/download interface bitstream file
	char* flashConfigBitstreamFile;	// configure FPGA from flash bitstream file
	char* testIntfcBitstreamFile;	// test interface bitstream file
	char* testBitstreamFile;		// board test bitstream file
	char* testObjFile;				// board test program object file
} XSBoardInfo; 


/* begin utils functions */

/* function: ConvertToUpperCase */
// convert a string to upper case
string& ConvertToUpperCase(string& s);

/* function: StripSuffix */
// strip suffix from a file name
string& StripSuffix(string& fileName);

/* function: GetSuffix */
// get suffix from a file name
string GetSuffix(string& fileName);

/* function: StripPrefix */
// strip prefixed directory path from a file name
string& StripPrefix(string& fileName);

/* function: GetPrefix */
// get prefixed directory path from a file name
string GetPrefix(string& fileName);

/* function: InsertDelay */
// subroutine for delaying by a given number of microseconds or milliseconds
void InsertDelay(unsigned long d, unsigned int time_units);

/* function: EnableBatch */
// subroutine for enabling/disabling prompts to user
void EnableBatch(bool b);

/* function: PromptUser */
// subroutine for prompting the user for a response
int PromptUser(string& msg, int action);

/* function: FindXSTOOLSBinDir */
// get the directory where the XSTOOL utilities are stored
const char* FindXSTOOLSBinDir(void);

/* function: SetXSTOOLSParameter */
// Set XSTOOLS parameter value
bool SetXSTOOLSParameter(string& name, string& value);

/* function: GetXSTOOLSParameter */
// Get XSTOOLS parameter value
string GetXSTOOLSParameter(string& name);

/* function: GetXSBoardInfo */
// get the information about the XS Boards from a file
int GetXSBoardInfo(XSBoardInfo** bInfo);

/* end utils functions */

#endif
