/*
 * This software is in the public domain.
 * You may use it as you see fit with the understanding that
 * we make no guarantees of its fitness for any particular purpose.
 *
 * Author: XESS Corporation
 *         www.xess.com
 */

//
// XC2SPort objects specialize a CnfgPort to handle programming of Spartan2 FPGAs
//

// $Log: xc2sprt.h $
// Revision 1.1  2001/12/30 15:48:15  DEVB
// Initial revision
//


#ifndef XC2SPort_H
#define XC2SPort_H


#include "Progress.h"
#include "CnfgPort.h"


class XC2SPort : public CnfgPort
{
	public:

	/* begin XC2SPort methods */

	/* method: XC2SPort */
	// Instantiate an XC2SPort object on a given parallel port
	XC2SPort(void);

	/* method: XC2SPort */
	// Instantiate an XC2SPort object on a given parallel port
	XC2SPort(XSError* e, // pointer to error reporting object
		unsigned int portNum, // parallel port number
		unsigned int invMask, // inversion mask for the parallel port
		unsigned int posCCLK, // bit position in parallel port of CCLK pin
		unsigned int posPROG, // bit position in parallel port of PROGRAM pin
		unsigned int posDlo, // lower bit position in parallel port of config. data pins
		unsigned int posDhi, // upper bit position in parallel port of config. data pins
		unsigned int posDONE // bit position in parallel port of DONE pin
		);

	/* method: Setup */
	// Setup an XC2SPort object on a given parallel port
	bool Setup(XSError* e, // pointer to error reporting object
		unsigned int portNum, // parallel port number
		unsigned int invMask, // inversion mask for the parallel port
		unsigned int posCCLK, // bit position in parallel port of CCLK pin
		unsigned int posPROG, // bit position in parallel port of PROGRAM pin
		unsigned int posDlo, // lower bit position in parallel port of config. data pins
		unsigned int posDhi, // upper bit position in parallel port of config. data pins
		unsigned int posDONE // bit position in parallel port of DONE pin
		);

	/* method: GetChipType */
	// field type for the FPGA device identifier
	// returns the chip identifier from the bitstream
	string GetChipType(istream& is);

	/* method: GetChipType */
	// returns the chip identifier from the bitstream
	string GetChipType(string& bitfileName);

	/* method: InitConfigureFPGA */
	// Initialize FPGA configuration pins
	void InitConfigureFPGA(void);

	/* method: EnableFastDownload */
	// enable or disable fast Spartan2 bitstream downloads
	void EnableFastDownload(bool t);

	/* method: ConfigureFPGA */
	// Send out a byte of configuration information
	void ConfigureFPGA(unsigned char b);

	/* method: ConfigureFPGA */
	// field type for the bitstream data
	// process a stream and send the configuration bitstream to the board
	bool ConfigureFPGA(istream& is);

	/* method: ConfigureFPGA */
	// configure the FPGA with the bit stream stored in a BIT file
	bool ConfigureFPGA(string& fileName);

	/* end XC2SPort methods */


	protected:

	Progress *progressGauge;	// indicates progress of operations
	unsigned int posDLO;		// lower bit position in parallel port of config. data pins
	unsigned int posDHI;		// upper bit position in parallel port of config. data pins


	private:

	string chipType;		// type of chip (e.g. 4005XLPC84)
	bool fastDownload;		// enable fast downloading of Spartan2 bitstreams

};

/* begin XC2SPort functions */

/* end XC2SPort functions */

#endif
