/*
 * This software is in the public domain.
 * You may use it as you see fit with the understanding that
 * we make no guarantees of its fitness for any particular purpose.
 *
 * Author: XESS Corporation
 *         www.xess.com
 */

//
// XC4KPort objects specialize a CnfgPort to handle programming of XC4000 FPGAs
//

// $Log: xc4kprt.h $
// Revision 1.1  2001/12/30 15:49:25  DEVB
// Initial revision
//


#ifndef XC4KPort_H
#define XC4KPort_H


#include "Progress.h"
#include "CnfgPort.h"


class XC4KPort : public CnfgPort
{
	public:

	/* begin XC4KPort methods */

	/* method: XC4KPort */
	// Instantiate an XC4KPort object on a given parallel port
	XC4KPort(void);

	/* method: XC4KPort */
	// Instantiate an XC4KPort object on a given parallel port
	XC4KPort(XSError* e, // pointer to error reporting object
		unsigned int portNum, // parallel port number
		unsigned int invMask, // inversion mask for the parallel port
		unsigned int posCCLK, // bit position in parallel port of CCLK pin
		unsigned int posPROG, // bit position in parallel port of PROGRAM pin
		unsigned int posDIN, // bit position in parallel port of DIN pin
		unsigned int posDONE // bit position in parallel port of DONE pin
		);

	/* method: Setup */
	// Setup an XC4KPort object on a given parallel port
	bool Setup(XSError* e, // pointer to error reporting object
		unsigned int portNum, // parallel port number
		unsigned int invMask, // inversion mask for the parallel port
		unsigned int posCCLK, // bit position in parallel port of CCLK pin
		unsigned int posPROG, // bit position in parallel port of PROGRAM pin
		unsigned int posDIN, // bit position in parallel port of DIN pin
		unsigned int posDONE // bit position in parallel port of DONE pin
		);

	/* method: GetChipType */
	// field type for the FPGA device identifier
	// returns the chip identifier from the bitstream
	string GetChipType(istream& is);

	/* method: GetChipType */
	// returns the chip identifier from the bitstream
	string GetChipType(string& bitfileName);

	/* method: InitConfigureFPGA */
	// Initialize FPGA configuration pins
	void InitConfigureFPGA(void);

	/* method: ConfigureFPGA */
	// Send out a byte of configuration information
	void ConfigureFPGA(unsigned char b);

	/* method: ConfigureFPGA */
	// field type for the bitstream data
	// process a stream and send the configuration bitstream to the board
	bool ConfigureFPGA(istream& is);

	/* method: ConfigureFPGA */
	// configure the FPGA with the bit stream stored in a BIT file
	bool ConfigureFPGA(string& fileName);

	/* end XC4KPort methods */


	protected:

	Progress *progressGauge;	// indicates progress of operations
	

	private:

	string chipType;		// type of chip (e.g. 4005XLPC84)

};

/* begin XC4KPort functions */

/* end XC4KPort functions */

#endif
