/*
 * This software is in the public domain.
 * You may use it as you see fit with the understanding that
 * we make no guarantees of its fitness for any particular purpose.
 *
 * Author: XESS Corporation
 *         www.xess.com
 */

//
// The XC95KPort object specializes the JTAGPort object so that the CPLD
// can be configured.
//

// $Log: xc95kprt.h $
// Revision 1.1  2001/12/30 15:51:28  DEVB
// Initial revision
//

#ifndef XC95KPort_H
#define XC95KPort_H

#include "Progress.h"
#include "JTAGPort.h"

class XC95KPort : public JTAGPort
{
	public:

	/* begin XC95KPort methods */

	/* method: XC95KPort */
	// Instantiate an object for an XC9500 CPLD connected to a parallel port
	XC95KPort(void);

	/* method: XC95KPort */
	// Instantiate an object for an XC9500 CPLD connected to a parallel port
	XC95KPort(XSError* err, // pointer to error reporting object
		unsigned int portNum, // parallel port number
		unsigned int invMask, // inversion mask for the parallel port
		unsigned int posTCK, // bit position in parallel port of TCK pin
		unsigned int posTMS, // bit position in parallel port of TMS pin
		unsigned int posTDI, // bit position in parallel port of TDI pin
		unsigned int posTDO); // bit position in parallel port of TDO pin

	/* method: Setup */
	// Setup an object for an XC9500 CPLD connected to a parallel port
	bool Setup(XSError* err, // pointer to error reporting object
		unsigned int portNum, // parallel port number
		unsigned int invMask, // inversion mask for the parallel port
		unsigned int posTCK, // bit position in parallel port of TCK pin
		unsigned int posTMS, // bit position in parallel port of TMS pin
		unsigned int posTDI, // bit position in parallel port of TDI pin
		unsigned int posTDO); // bit position in parallel port of TDO pin

	/* method: GetChipID */
	// detect the presence and type of the XC9500 CPLD chip on the port
	string GetChipID();

	/* method: GetUSERCODE */
	// read the 32-bit USERCODE from the XC9500 CPLD chip
	string GetUSERCODE();

	/* method: InitConfigureCPLD */
	// Initialize the XC9500 CPLD for configuration through the JTAG interface
	void InitConfigureCPLD(void);

	/* method: ConfigureCPLD */
	// Send out a byte of configuration information to the XC9500 Flash
	void ConfigureCPLD(unsigned char b);

	/* method: ConfigureCPLD */
	// Process SVF and send results through JTAG configuration pins
	bool ConfigureCPLD(istream& is);

	/* method: ConfigureCPLD */
	// configure the CPLD with the bit stream stored in an SVF file
	bool ConfigureCPLD(string& fileName);

	/* end XC95KPort methods */


	protected:

	Progress *progressGauge;	// indicates progress of operations


	private:

	string chipType;		// type of chip (e.g. XC95108)

};

/* begin XC95KPort functions */

/* function: NextWord */
static string NextWord(string& s);

/* end XC95KPort functions */

#endif
