/*
 * This software is in the public domain.
 * You may use it as you see fit with the understanding that
 * we make no guarantees of its fitness for any particular purpose.
 *
 * Author: XESS Corporation
 *         www.xess.com
 */

//
// The XCBSDR object specializes a bitstream into a boundary scan data register
// that is used to perform RAM operations via the JTAG port.

// $Log: xcbsdr.h $
// Revision 1.3  1999/06/29 16:53:37  DEVB
// changed number of RAM address bits to 17 to support 128 KByte RAMs
// on the XS Boards.
//
// Revision 1.2  1999/03/30 18:27:41  DEVB
// Released source into the public domain
//
// Revision 1.1  1999/03/29 22:56:56  DEVB
// Initial revision
//

#ifndef XCBSDR_H
#define XCBSDR_H

#include "bitstrm.h"

const int numRAMAddressBits = 17;
const int numRAMDataBits = 8;
const int numRAMControlBits = 3;

class XCBSDR : public Bitstream
{
public:
	
	XCBSDR(unsigned int length,
		unsigned int *addressBitPos,
		unsigned int *dataBitPos,
		unsigned int *controlBitPos);
	
	// place a RAM address into the bitstream buffer
	void SetRAMAddress(unsigned int address);
	
	// place RAM data into the bitstream buffer
	void SetRAMData(unsigned int data);
	
	// tristate the FPLD pins so the RAM data pins can be read
	void ReadRAMData(void);
	
	// get the RAM data from the bsdr bitstream
	unsigned int GetRAMData(void);
	
	// get the RAM address from the bsdr bitstream
	unsigned long GetRAMAddress(void);
	
	// place RAM control pin levels into the bsdr bitstream
	void SetRAMControls(unsigned int cs_,
		unsigned int oe_,
		unsigned int we_);
	
private:
	// The RAM of the XS board is controlled through a JTAG interface,
	// and these arrays hold the positions of the RAM pins in the data register
	// position of RAM address bits in data register
	unsigned int ramAddressBitPos[numRAMAddressBits];
	// position of RAM data bits in data register
	unsigned int ramDataBitPos[numRAMDataBits];
	// position of RAM control bits in data register
	unsigned int ramControlBitPos[numRAMControlBits];
};

#endif
