/*
 * This software is in the public domain.
 * You may use it as you see fit with the understanding that
 * we make no guarantees of its fitness for any particular purpose.
 *
 * Author: XESS Corporation
 *         www.xess.com
 */

//
// This object loads the FPGA or Flash with a bitstream,
// uploads/downloads the RAM, sets the oscillator frequency,
// and tests the board.
//

// $Log: xs40brd.h $
// Revision 1.1  2001/12/30 15:52:35  DEVB
// Initial revision
//

#ifndef XS40Board_H
#define XS40Board_H

#include "XSBoard.h"
#include "XC4KPrt.h"
#include "OscPort.h"
#include "RAMPort.h"
#include "AT17Prt.h"

class XS40Board : public XSBoard
{
	public:

	/* begin XS40Board methods */

	/* method: XS40Board */
	// create an object for programming the XS40 Board
	 XS40Board(void);

	/* method: Setup */
	// setup the object for programming the XS40 Board
	bool Setup(XSError* err,
		const char* brdModel,		// model of XS40 Board
		unsigned int lptNum
		);

	/* method: Configure */
	// load a configuration bitstream into the FPGA or Flash
	bool Configure(
		string& fileName // bitstream file
		);

	/* method: ConfigureInterface */
	// load a configuration bitstream into the interface FPGA
	bool ConfigureInterface(
		string& fileName // bitstream file
		);

	/* method: DownloadRAM */
	// download into RAM
	virtual bool DownloadRAM(
		string& fileName,		// RAM data file
		bool doStart,			// perform startup operations for RAM downloads
		bool doEnd				// perform terminating operations for RAM downloads
		);

	/* method: UploadRAM */
	// upload RAM from loAddr to hiAddr into file
	virtual bool UploadRAM(
		string& fileName, // RAM data file
		const char* format,		// format - Intel hex, Moto exo, or XESS
		unsigned int loAddr, // lower address of RAM data
		unsigned int hiAddr // upper address of RAM data
		);

	/* method: DownloadFlash */
	// download into Flash
	virtual bool DownloadFlash(
		string& fileName,		// Flash data file
		bool doStart,			// perform startup operations for Flash downloads
		bool doEnd				// perform terminating operations for Flash downloads
		);

	/* method: UploadFlash */
	// upload Flash from loAddr to hiAddr into file
	virtual bool UploadFlash(
		string& fileName, // Flash data file
		const char* format,		// format - Intel hex, Moto exo, or XESS
		unsigned int loAddr, // lower address of RAM data
		unsigned int hiAddr // upper address of RAM data
		);

	/* method: SetFreq */
	// program the oscillator frequency
	bool SetFreq(
		int div, // divisor for osc. frequency
		bool extOscPresent // true if external clock drives oscillator
		);

	/* method: Test */
	// test the XS40 Board
	bool Test(
		void
		);

	/* end XS40Board methods */

	private:

	char* brdModel;		// model of XS40 Board
	XC4KPort fpga;
	OscPort osc;
	RAMPort ram;
	AT17Port seeprom;
};

/* begin XS40Board functions */

/* end XS40Board functions */

#endif
