/*
 * This software is in the public domain.
 * You may use it as you see fit with the understanding that
 * we make no guarantees of its fitness for any particular purpose.
 *
 * Author: XESS Corporation
 *         www.xess.com
 */

//
// This object loads the FPGA or Flash with a bitstream,
// uploads/downloads the RAM, sets the oscillator frequency,
// and tests the board.
//

// $Log: xs95brd.h $
// Revision 1.1  2001/12/30 15:55:41  DEVB
// Initial revision
//

#ifndef XS95Board_H
#define XS95Board_H

#include "XSBoard.h"
#include "XC95KPrt.h"
#include "OscPort.h"
#include "JRAMPrt.h"

class XS95Board : public XSBoard
{
	public:

	/* begin XS95Board methods */

	/* method: XS95Board */
	// create an object for programming the XS95 Board
	 XS95Board(void);

	/* method: Setup */
	// setup the object for programming the XS95 Board
	bool Setup(XSError* err,
		const char* brdModel,		// model of XS95 Board
		unsigned int lptNum
		);

	/* method: CheckChipID */
	// check the chip ID and make sure it is an XC95108
	bool CheckChipID(void);

	/* method: Configure */
	// load a configuration bitstream into the FPGA or Flash
	bool Configure(
		string& fileName // bitstream file
		);

	/* method: ConfigureInterface */
	// load a configuration bitstream into the interface FPGA
	bool ConfigureInterface(
		string& fileName // bitstream file
		);

	/* method: DownloadRAM */
	// download into RAM
	virtual bool DownloadRAM(
		string& fileName,		// RAM data file
		bool doStart,			// perform startup operations for RAM downloads
		bool doEnd				// perform terminating operations for RAM downloads
		);

	/* method: UploadRAM */
	// upload RAM from loAddr to hiAddr into file
	virtual bool UploadRAM(
		string& fileName, // RAM data file
		const char* format,		// format - Intel hex, Moto exo, or XESS
		unsigned int loAddr, // lower address of RAM data
		unsigned int hiAddr // upper address of RAM data
		);

	/* method: DownloadFlash */
	// download into Flash
	virtual bool DownloadFlash(
		string& fileName,		// Flash data file
		bool doStart,			// perform startup operations for Flash downloads
		bool doEnd				// perform terminating operations for Flash downloads
		);

	/* method: UploadFlash */
	// upload Flash from loAddr to hiAddr into file
	virtual bool UploadFlash(
		string& fileName, // Flash data file
		const char* format,		// format - Intel hex, Moto exo, or XESS
		unsigned int loAddr, // lower address of RAM data
		unsigned int hiAddr // upper address of RAM data
		);

	/* method: SetFreq */
	// program the oscillator frequency
	bool SetFreq(
		int div, // divisor for osc. frequency
		bool extOscPresent // true if external clock drives oscillator
		);

	/* method: Test */
	// test the XS95 Board
	bool Test(
		void
		);

	/* end XS95Board methods */

	private:

	char* brdModel;		// model of XS95 Board
	XC95KPort cpld;
	OscPort osc;
	JTAGRAMPort ram;
};

/* begin XS95Board functions */

/* end XS95Board functions */

#endif
