/*
 * This software is in the public domain.
 * You may use it as you see fit with the understanding that
 * we make no guarantees of its fitness for any particular purpose.
 *
 * Author: XESS Corporation
 *         www.xess.com
 */

//
// This object loads the FPGA or Flash with a bitstream,
// uploads/downloads the RAM, sets the oscillator frequency,
// and tests the board.
//

// $Log: xsaboard.h $
// Revision 1.1  2001/12/30 15:57:50  DEVB
// Initial revision
//

#ifndef XSABoard_H
#define XSABoard_H

#include "Utils.h"
#include "XSBoard.h"
#include "XC2SPrt.h"
#include "XC95KPrt.h"
#include "OscPort.h"
#include "RAMPort.h"
#include "AT49FPrt.h"

class XSABoard : public XSBoard
{
	public:

	/* begin XSABoard methods */

	/* method: XSABoard */
	// create an object for programming the XSA Board
	 XSABoard(void);

	/* method: Setup */
	// setup the object for programming the XSA Board
	bool Setup(XSError* err,
		const char* brdModel,		// model of XSA Board
		unsigned int lptNum
		);

	/* method: CheckChipID */
	// check the interface chip ID and make sure it is an XC9572XL
	bool CheckChipID(void);

	/* method: Configure */
	// load a configuration bitstream into the FPGA or Flash
	bool Configure(
		string& fileName // bitstream file
		);

	/* method: ConfigureInterface */
	// load a configuration bitstream into the interface FPGA
	bool ConfigureInterface(
		string& fileName // bitstream file
		);

	/* method: DownloadRAM */
	// download into RAM
	virtual bool DownloadRAM(
		string& fileName,		// RAM data file
		bool doStart,			// perform startup operations for RAM downloads
		bool doEnd				// perform terminating operations for RAM downloads
		);

	/* method: UploadRAM */
	// upload RAM from loAddr to hiAddr into file
	virtual bool UploadRAM(
		string& fileName,		// RAM data file
		const char* format,		// format - Intel hex, Moto exo, or XESS
		unsigned int loAddr,	// lower address of RAM data
		unsigned int hiAddr		// upper address of RAM data
		);

	/* method: DownloadFlash */
	// download into Flash
	virtual bool DownloadFlash(
		string& fileName,		// Flash data file
		bool doStart,			// perform startup operations for Flash downloads
		bool doEnd				// perform terminating operations for Flash downloads
		);

	/* method: UploadFlash */
	// upload Flash from loAddr to hiAddr into file
	virtual bool UploadFlash(
		string& fileName, // Flash data file
		const char* format,		// format - Intel hex, Moto exo, or XESS
		unsigned int loAddr, // lower address of RAM data
		unsigned int hiAddr // upper address of RAM data
		);

	/* method: SetFreq */
	// program the oscillator frequency
	bool SetFreq(
		int div, // divisor for osc. frequency
		bool extOscPresent // true if external clock drives oscillator
		);

	/* method: Test */
	// test the XSA Board
	bool Test(
		void
		);

	/* end XSABoard methods */

	private:

	char* brdModel;		// model of XSA Board
	XC2SPort fpga;
	XC95KPort cpld;
	OscPort osc;
	RAMPort ram;
	AT49FPort flash;
};

/* begin XSABoard functions */

/* end XSABoard functions */

#endif
