/*
 * This software is in the public domain.
 * You may use it as you see fit with the understanding that
 * we make no guarantees of its fitness for any particular purpose.
 *
 * Author: XESS Corporation
 *         www.xess.com
 */

//
// Generic XS Board object
//

// $Log: xsboard.h $
// Revision 1.1  2001/12/30 16:27:10  DEVB
// Initial revision
//

#ifndef XSBoard_H
#define XSBoard_H

#include <string>
using std::string;
#include "XSError.h"


class XSBoard
{
	public:

	/* begin XSBoard methods */

	/* method: XSBoard */
	// create an object for programming the XSA Board
//	 XSBoard(void);

	/* method: Setup */
	// setup the object for programming the XSA Board
	virtual bool Setup(XSError* err,
		const char* brdModel,		// model of XSV Board
		unsigned int lptNum
		) = 0;

	/* method: Configure */
	// load a configuration bitstream into the FPGA or Flash
	virtual bool Configure(
		string& fileName // bitstream file
		) = 0;

	/* method: ConfigureInterface */
	// load a configuration bitstream into the interface FPGA
	virtual bool ConfigureInterface(
		string& fileName // bitstream file
		) = 0;

	/* method: DownloadRAM */
	// download into RAM
	virtual bool DownloadRAM(
		string& fileName,		// RAM data file
		bool doStart,			// perform startup operations for RAM downloads
		bool doEnd				// perform terminating operations for RAM downloads
		) = 0;

	/* method: UploadRAM */
	// upload RAM from loAddr to hiAddr into file
	virtual bool UploadRAM(
		string& fileName, // RAM data file
		const char* format,		// format - Intel hex, Moto exo, or XESS
		unsigned int loAddr, // lower address of RAM data
		unsigned int hiAddr // upper address of RAM data
		) = 0;

	/* method: DownloadFlash */
	// download into Flash
	virtual bool DownloadFlash(
		string& fileName,		// Flash data file
		bool doStart,			// perform startup operations for Flash downloads
		bool doEnd				// perform terminating operations for Flash downloads
		) = 0;

	/* method: UploadFlash */
	// upload Flash from loAddr to hiAddr into file
	virtual bool UploadFlash(
		string& fileName, // Flash data file
		const char* format,		// format - Intel hex, Moto exo, or XESS
		unsigned int loAddr, // lower address of RAM data
		unsigned int hiAddr // upper address of RAM data
		) = 0;

	/* method: SetFreq */
	// program the oscillator frequency
	virtual bool SetFreq(
		int div, // divisor for osc. frequency
		bool extOscPresent // true if external clock drives oscillator
		) = 0;

	/* method: Test */
	// test the XSA Board
	virtual bool Test(
		void
		) = 0;

	/* end XSBoard methods */

};

/* begin XSBoard functions */

/* end XSBoard functions */

#endif
